/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GRAPHICPRIMITIVE_H
#define GRAPHICPRIMITIVE_H

#include "matrix4x4.h"
#include "matrix.h"

// Forward declarations
class QImage;
class QPointF;
class QColor;
class ScanLineTable;
class Point4F;
class ScanLineTable;

/// This class is the base of all primitives
/**
	This class is the base of all the figures we implement.
*/

class GraphicPrimitive
{
public:

	GraphicPrimitive();
	/// Draw the primitive
	/**
		As this function is virtual, all the descending classes must implement it.
	*/
	virtual void draw(const Matrix4X4 & incomingMatrix, Matrix * zbuffer,
	                  bool perspective, QImage * image) = 0 ;

	/// Get the x displacement
	double getXDisplacement();

	/// Get the y displacement
	double getYDisplacement();

	/// Get the z displacement
	double getZDisplacement();

	/// Get the displacement matrix
	Matrix4X4 getDisplacementMatrix();

	/// Get the rotation angle
	double getAngleOfRotationX();

	/// Get the rotation angle
	double getAngleOfRotationY();

	/// Get the rotation angle
	double getAngleOfRotationZ();

	/// Get the rotation matrix
	Matrix4X4 getRotationMatrix();

	/// Get the x scale
	double getXScale();

	/// Get the y scale
	double getYScale();

	/// Get the z scale
	double getZScale();

	/// Get the scale matrix
	Matrix4X4 getScaleMatrix();

	/// Get the transformation matrix
	Matrix4X4 getTransformationMatrix();

	/// Set the displacement in the x coordinate
	void setXDisplacement(double displacement);

	/// Set the displacement in the y coordinate
	void setYDisplacement(double displacement);

	/// Set the displacement in the z coordinate
	void setZDisplacement(double displacement);

	/// Set the angle of rotation
	void setAngleOfRotationX(double theta);

	/// Set the angle of rotation
	void setAngleOfRotationY(double theta);

	/// Set the angle of rotation
	void setAngleOfRotationZ(double theta);

	/// Set the scalement in x
	void setXScale(double scale);

	/// Set the scalement in y
	void setYScale(double scale);

	/// Set the scalement in z
	void setZScale(double scale);

protected:
	/// Draws a line that takes into account the Z-Buffer
	void drawZBufferLine(Point4F begin, Point4F end,
                       const QColor firstColor, const QColor secondColor,
                       ScanLineTable & table, Matrix * zbuffer,
                       bool perspective, QImage * image);
	/// Interpolates colors
	QColor interpolateColors(const QColor firstColor, const QColor secondColor,
	                         const int begin, const int end, const int position);

	/// Interpolates the value of z
	double interpolateZ(const Point4F & firstPoint, const Point4F & secondPoint,
	                    const int begin, const int end, const int position);

	/// Draws the scan line table
	void drawScanLineTable(ScanLineTable & table, Matrix * zbuffer,
	                       bool perspective, QImage * image);

private:
	double xDisplacement;
	double yDisplacement;
	double zDisplacement;
	double angleOfRotationX;
	double angleOfRotationY;
	double angleOfRotationZ;
	double xScale;
	double yScale;
	double zScale;
};

#endif // GraphicPrimitive



